//+------------------------------------------------------------------+
//| VortexIndicator_TRO_MODIFIED_VERSION       Vortex Indicator.mq4  |
//|From the January 2010 issue of Technical Analysis of Stocks &     |
//|Commodities                                                       |
//+------------------------------------------------------------------+
//| VortexIndicator_TRO_MODIFIED_VERSION                             |
//| MODIFIED BY AVERY T. HORTON, JR. AKA THERUMPLEDONE@GMAIL.COM     |
//| I am NOT the ORIGINAL author 
//  and I am not claiming authorship of this indicator. 
//  All I did was modify it. I hope you find my modifications useful.|
//|                                                                  |
//+------------------------------------------------------------------+
 

#property  copyright "Copyright 2009 under Creative Commons BY-SA License by Neil D. Rosenthal"
#property  link      "http://creativecommons.org/licenses/by-sa/3.0/"

#property indicator_separate_window
#property indicator_buffers 8
#property indicator_color1 Green
#property indicator_color2 Red
#property indicator_color8 Blue

//---- Input parameters
extern bool   Sound.Alert    = false ;
extern bool   Send.Mail      = false ;
extern bool   Show.Comment   = false ;
extern bool   Show.Legend    = false ; 


extern int VI_Length=14;

extern double iMaxThreshold = 0.80 ;
extern double iMinThreshold = -0.80 ;

//---- Buffers
double PlusVI[];        //VI+ : + Vortex Indicator buffer
double MinusVI[];       //VI- : - Vortex Indicator buffer
double PlusVM[];        //VM+ : + Vortex Movement buffer
double MinusVM[];       //VM- : - Vorext Movement buffer
double SumPlusVM[];     //Sum of VI_Length PlusVM values
double SumMinusVM[];    //Sum of VI_Length MinusVM values
double SumTR[];         //True Range buffer
double DiffBuf[];         //  buffer

//+------------------------------------------------------------------+
string   TAG, symbol, tChartPeriod,  tShortName ;  
double   point, diff, maxdiff=-999, mindiff=999;
int      digits, period, win , CrossUp, CrossDn, MaxThresholdCount, MinThresholdCount, minbar, maxbar ;  
bool     bCrossUp, bCrossDn ; 
datetime Trigger ;
//+------------------------------------------------------------------+
int init()
{
   period       =  Period() ;    
   symbol       =  Symbol() ;
   digits       =  Digits ;   
   point        =  Point ; 
   
   tChartPeriod =  TimeFrameToString(period) ;  
   
   TAG          =  "VORTEX" ;
   tShortName   =  "VORTEX("+ VI_Length + ")";
   
   IndicatorShortName(tShortName);
   
//----

    IndicatorDigits(Digits);
//---- Set visible buffer properties    
    SetIndexBuffer(0,PlusVI);
    SetIndexBuffer(1,MinusVI);
    SetIndexBuffer(7,DiffBuf);
        
    SetIndexStyle(0,DRAW_NONE);    
    SetIndexStyle(1,DRAW_NONE);
    SetIndexStyle(2,DRAW_NONE);    
    SetIndexStyle(3,DRAW_NONE);    
    SetIndexStyle(4,DRAW_NONE);    
    SetIndexStyle(5,DRAW_NONE);     
    SetIndexStyle(6,DRAW_NONE);    
    SetIndexStyle(7,DRAW_HISTOGRAM); //SetIndexStyle(7,DRAW_HISTOGRAM,0,1,Blue); 
        
    SetIndexLabel(0,"PlusVI(" + VI_Length + ")");
    SetIndexLabel(1,"MinusVI(" + VI_Length + ")");
    SetIndexDrawBegin(0,VI_Length);
    SetIndexDrawBegin(1,VI_Length);
    SetIndexDrawBegin(7,VI_Length);    
//---- Set indices of caching buffers
    SetIndexBuffer(2,PlusVM);
    SetIndexBuffer(3,MinusVM);
    SetIndexBuffer(4,SumPlusVM);
    SetIndexBuffer(5,SumMinusVM);
    SetIndexBuffer(6,SumTR);
     


   SetLevelStyle(STYLE_DOT, 1, LightGray) ;
   SetLevelValue(1,iMaxThreshold ); 
   SetLevelValue(2,iMinThreshold );   

    
//----

 

   return(0);
}
//+------------------------------------------------------------------+
 
void ObDeleteObjectsByPrefix(string Prefix)
  {
   int L = StringLen(Prefix);
   int i = 0; 
   while(i < ObjectsTotal())
     {
       string ObjName = ObjectName(i);
       if(StringSubstr(ObjName, 0, L) != Prefix) 
         { 
           i++; 
           continue;
         }
       ObjectDelete(ObjName);
     }
  }
     
//+------------------------------------------------------------------+
int deinit()
{
//----
   ObDeleteObjectsByPrefix(TAG) ;
   Comment("");   
//----
    return(0);
}
 
//+------------------------------------------------------------------+
int start()
{
    win =    WindowFind(tShortName);

    int Limit;
    int CountedBars = IndicatorCounted();
//---- Check for possible errors
    if(CountedBars < 0) return(-1);
//---- Last counted bar will be recounted
    if(CountedBars > 0) CountedBars--;
    Limit = Bars - CountedBars;
//---- Clear caching buffers
    for(int i = 0; i < Limit; i++)
    {
      SumPlusVM[i] = 0;
      SumMinusVM[i] = 0;
      SumTR[i]= 0;  
    }
//---- Store the values of PlusVM and MinusVM
    for(i = 0; i < Limit; i++)
    {
        //PlusVM = |Today's High - Yesterday's Low|
        PlusVM[i] = MathAbs(High[i] - Low[i + 1]);
        //MinusVM = |Today's Low - Yesterday's High|
        MinusVM[i] = MathAbs(Low[i] - High[i +1]);  
    }
//---- Sum VI_Length values of PlusVM, MinusVM and the True Range


    for(i = 0; i < Limit; i++)
    {
        for(int j = 0; j <= VI_Length - 1; j++)
        {
           SumPlusVM[i] += PlusVM[i + j];
           SumMinusVM[i] += MinusVM[i + j];
           SumTR[i] += iATR(NULL,0,1,i + j); //Sum VI_Length values of the True Range by using a 1-period ATR
        }
    }
//---- Draw the indicator



    for(i = 0; i < Limit; i++)
    {
        PlusVI[i]  = SumPlusVM[i] / SumTR[i];
        MinusVI[i] = SumMinusVM[i] / SumTR[i];
        diff       = PlusVI[i] - MinusVI[i] ; 
        DiffBuf[i] = diff ;
        
    if( diff > maxdiff ) { maxdiff = diff ; maxbar = i ;}
    if( diff < mindiff ) { mindiff = diff ; minbar = i ;}
    
    if( diff >= iMaxThreshold ) { MaxThresholdCount = MaxThresholdCount + 1 ;}
    if( diff <= iMinThreshold ) { MinThresholdCount = MinThresholdCount + 1 ;}
      

    }
//----

 bCrossUp = false ;   
 bCrossDn = false ; 

CrossUp = -1 ;
CrossDn = -1 ; 

for( i=0; i<1000 ; i++ )
{
   if( PlusVI[i] > MinusVI[i] && PlusVI[i+1] < MinusVI[i+1] && !bCrossUp ) {bCrossUp = true ; CrossUp = i ;}
   if( PlusVI[i] < MinusVI[i] && PlusVI[i+1] > MinusVI[i+1] && !bCrossDn ) {bCrossDn = true ; CrossDn = i ;}
   if( bCrossUp && bCrossDn) { break ; }   
} 



if(Show.Comment )
{
 
Comment("\n", 
    
  
"Last Vortex Cross Up = ", TimeToStr(Time[CrossUp]) ,  "\n",   
"Last Vortex Cross Dn = ", TimeToStr(Time[CrossDn]) ,  "\n", 
"Vortex Maximum       = ", maxdiff , " at ", TimeToStr(Time[maxbar]) , "\n", 
"Vortex Minimum       = ", mindiff , " at ", TimeToStr(Time[minbar]) , "\n",    
"Threshold            = ", iMaxThreshold , " exceeded ", MaxThresholdCount, " times" , "\n",  
"Threshold            = ", iMinThreshold , " exceeded ", MinThresholdCount, " times" , "\n",   
"") ;    
   
}// if

  if ( Trigger != Time[0] ) 
  {
  if( Sound.Alert )
  {
        if( CrossUp == 0 ) { Trigger = Time[0] ; Alert(symbol,"  ", tChartPeriod, " Vortex Cross Up"+ DoubleToStr(Close[0] ,digits)); }
        if( CrossDn == 0 ) { Trigger = Time[0] ; Alert(symbol,"  ", tChartPeriod, " Vortex Cross Down "+ DoubleToStr(Close[0] ,digits)); }     
  }
  
  if( Send.Mail )
  {
     if( CrossUp == 0 ) { Trigger = Time[0] ; SendMail(symbol,  tChartPeriod + " Vortex Cross Up "); }
     if( CrossDn == 0 ) { Trigger = Time[0] ; SendMail(symbol,  tChartPeriod + " Vortex Cross Down "); }     
  }  
 
  }

    if(Show.Legend) { DoShowLegend(); }   
          
    WindowRedraw() ; 

    return(0);
}

//+------------------------------------------------------------------+  
void DoShowLegend()
{ 

   ObDeleteObjectsByPrefix(TAG);
   int yInc , yAdd ;

   yInc = 12 ; 
   yAdd = 13;

   setObject(TAG+"ss",symbol    ,100,yInc, Gold ); 
   setObject(TAG+"ss1",DoubleToStr(Close[0],digits) ,11,yInc,Gold     ,"Courier New");

   yInc = yInc + yAdd;    
   setObject(TAG+"va","Current Value  " ,50,yInc, Gold );     
   setObject(TAG+"va1",DoubleToStr(DiffBuf[0],4) ,11,yInc,Gold     ,"Courier New");
   
   yInc = yInc + yAdd;    
   setObject(TAG+"up","Crossed Up     " ,50,yInc, Gold ); 
   yInc = yInc + yAdd;      
   setObject(TAG+"up1",TimeToStr(Time[CrossUp]) ,11,yInc,Gold     ,"Courier New");
   
   yInc = yInc + yAdd;      
   setObject(TAG+"dn","Crossed Down   " ,50,yInc, Gold); 
   yInc = yInc + yAdd;      
   setObject(TAG+"dn1",TimeToStr(Time[CrossDn]) ,11,yInc,Gold     ,"Courier New");
   
   yInc = yInc + yAdd; 
   setObject(TAG+"MX","Maximum Value  " ,50,yInc, Gold); setObject(TAG+"mx1",DoubleToStr( maxdiff , 4) ,11,yInc,Gold     ,"Courier New");

   yInc = yInc + yAdd; 
   //setObject(TAG+"Md","Maximum Date   " ,50,yInc, Gold); 
   setObject(TAG+"md1",TimeToStr(Time[maxbar]) ,11,yInc,Gold     ,"Courier New");

   yInc = yInc + yAdd; 
   setObject(TAG+"nX","Minimum Value  " ,50,yInc, Gold); setObject(TAG+"nx1",DoubleToStr( mindiff , 4) ,11,yInc,Gold     ,"Courier New");

   yInc = yInc + yAdd; 
   //setObject(TAG+"Md","Maximum Date   " ,50,yInc, Gold); 
   setObject(TAG+"nd1",TimeToStr(Time[minbar]) ,11,yInc,Gold     ,"Courier New");
 
   yInc = yInc + yAdd; 
   setObject(TAG+"MXt","Threshold      " ,50,yInc, Gold); setObject(TAG+"mxt1",DoubleToStr( iMaxThreshold , 4) ,11,yInc,Gold     ,"Courier New");
   yInc = yInc + yAdd; 
   setObject(TAG+"MXe","Exceeded       " ,50,yInc, Gold); setObject(TAG+"mxe1",MaxThresholdCount,11,yInc,Gold     ,"Courier New");
  
   yInc = yInc + yAdd; 
   setObject(TAG+"Mnt","Threshold      " ,50,yInc, Gold); setObject(TAG+"mnt1",DoubleToStr( iMinThreshold , 4) ,11,yInc,Gold     ,"Courier New");
   yInc = yInc + yAdd; 
   setObject(TAG+"Mne","Exceeded       " ,50,yInc, Gold); setObject(TAG+"mne1",MinThresholdCount,11,yInc,Gold     ,"Courier New");
       
}

//+------------------------------------------------------------------+  

void setObject(string labelName,string text,int x,int y,color theColor, string font = "Courier New",int size=10,int angle=0)
{
 
      
      if (ObjectFind(labelName) == -1)
          {
             ObjectCreate(labelName,OBJ_LABEL,win,0,0);
             ObjectSet(labelName,OBJPROP_CORNER,1);
             if (angle != 0)
                  ObjectSet(labelName,OBJPROP_ANGLE,angle);
          }               
       ObjectSet(labelName,OBJPROP_XDISTANCE,x);
       ObjectSet(labelName,OBJPROP_YDISTANCE,y);
       ObjectSetText(labelName,text,size,font,theColor);
}
//+------------------------------------------------------------------+
 

string TimeFrameToString(int tf)
{
   string tfs;
   switch(tf) {
      case PERIOD_M1:  tfs="M1"  ; break;
      case PERIOD_M5:  tfs="M5"  ; break;
      case PERIOD_M15: tfs="M15" ; break;
      case PERIOD_M30: tfs="M30" ; break;
      case PERIOD_H1:  tfs="H1"  ; break;
      case PERIOD_H4:  tfs="H4"  ; break;
      case PERIOD_D1:  tfs="D1"  ; break;
      case PERIOD_W1:  tfs="W1"  ; break;
      case PERIOD_MN1: tfs="MN";
   }
   return(tfs);
}
  
//+------------------------------------------------------------------+  

